; File:                          boot.asm
; Description:                  DOS-C boot
;
;                       Copyright (c) 1997;
;                           Svante Frey
;                       All Rights Reserved
;
; This file is part of DOS-C.
;
; DOS-C is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version
; 2, or (at your option) any later version.
;
; DOS-C is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
; the GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public
; License along with DOS-C; see the file COPYING.  If not,
; write to the Free Software Foundation, 675 Mass Ave,
; Cambridge, MA 02139, USA.
; ---------------------------------------------------------------------------
; In this revision and rewrite of the Freedos boot sector, I have:
;
;    Split the one boot sector program into two. (for easier managment)
;
;    Corrected startup problem, where if a freedos formated disk is in the
;    drive at startup without a operating system being present it freezes.
;    It will now returns a error message and awaits further instructions.
;
;    I have made an effort to completely fill the boot sector program
;    with necessary code and not leave any spaces for computer viruses
;    to utilize.

;    I have also cleaned up the streamlinde the code and made the error
;    message more informative. I've also removed the "ROOT", "FAT",
;    "KERNEL", and "GO" startup messages that clutter up the startup.
;   
;    Also note that there is a different method of displaying text in
;    each program.  The boot12.asm program has the standard dos method
;    whereas the boot16 uses the freedos method. (With the freedos method
;    problems can arise if the message it to long as the message is
;    contained within the code.  Other wise there's no real difference.
;
;    Brian Foster
;----------------------------------------------------------------------------
; $Log: boot.asm,v $
; Revision 1.3  2000/05/25 20:56:19  jimtabor
; Fixed project history
;
; Revision 1.2  2000/05/11 03:56:42  jimtabor
; Clean up and Release
;
; Revision 1.1.1.1  2000/05/06 19:34:53  jhall1
; The FreeDOS Kernel.  A DOS kernel that aims to be 100% compatible with
; MS-DOS.  Distributed under the GNU GPL.
;
; Revision 1.12  1999/09/25 06:42:18  jprice
; Optimize boot loader.  Documentation.
;
; Revision 1.11  1999/09/24 19:04:55  jprice
; Added changes recommended by Jens Horstmeier

; to make their bootable CD work.
;
; Revision 1.10  1999/09/23 04:39:02  jprice
; *** empty log message ***
;
; Revision 1.7  1999/04/23 03:43:46  jprice
; Ported to NASM by ror4
;
; Revision 1.6  1999/04/17 19:14:03  jprice
; Fixed multi-sector code
;
; Revision 1.5  1999/04/17 06:23:26  jprice
; Changed so multi-sector IO is optional.
;
; Revision 1.4  1999/04/13 15:52:22  jprice
; Moves boot sector to top of mem
;
; Revision 1.3  1999/04/06 22:53:36  jprice
; Put back code to read multiple sectors at a time.
;
; Revision 1.2  1999/04/01 07:23:20  jprice
; New boot loader
;
; Revision 1.1.1.1  1999/03/29 15:39:39  jprice
; New version without IPL.SYS
;
; Revision 1.3  1999/03/02 06:57:14  jprice
; Added entry address for more recent versions of TLINK
;
; Revision 1.2  1999/01/21 05:03:58  jprice
; Formating.
;
; Revision 1.1.1.1  1999/01/20 05:51:00  jprice
; Imported sources
;
;          Rev 1.5   10 Jan 1997  4:58:06   patv
;       Corrected copyright
;
;          Rev 1.4   10 Jan 1997  4:52:50   patv
;       Re-written to support C drive and eliminate restrictions on IPL.SYS
;
;          Rev 1.3   29 Aug 1996 13:06:50   patv
;       Bug fixes for v0.91b
;
;          Rev 1.2   01 Sep 1995 17:56:44   patv
;       First GPL release.
;
;          Rev 1.1   30 Jul 1995 20:37:38   patv
;       Initialized stack before use.
;
;          Rev 1.0   02 Jul 1995 10:57:52   patv
;       Initial revision.
;
;       +--------+
;       |        |
;       |        |
;       |--------| 4000:0000
;       |        |
;       |  FAT   |
;       |        |
;       |--------| 2000:0000
;       |BOOT SEC|
;       |RELOCATE|
;       |--------| 1FE0:0000
;       |        |
;       |        |
;       |        |
;       |        |
;       |--------|
;       |BOOT SEC|
;       |ORIGIN  | 07C0:0000
;       |--------|
;       |        |
;       |        |
;       |        |
;       |--------|
;       |KERNEL  |
;       |LOADED  |
;       |--------| 0060:0000
;       |        |
;       +--------+
;
;----------------------------------------------------------------------------
;
; START OF PROGRAM.
;
; The first 62 bytes are standard data format for the DOS boot sectors.
; The information is entered during formating and is a record of the disk's
; size and format.
;
; The first 3 bytes of the boot sector are reserved for the jump over data
; to the start ot the code sector.
;----------------------------------------------------------------------------
;
; The next 8 bytes are reserved for the ID of the OS that the boot sector
; is associated with. (Called in MS-DOS OEM label.)
;----------------------------------------------------------------------------
;
; The next 26 bytes are a record of the disks compacitity after formating.

BytesPerSector      equ    0Bh    ; DW   bytes/sector
SectorsPerCluster   equ    0Dh    ; DB   sectors/allocation unit
ReservedSectors     equ    0Eh    ; DW   # reserved sectors
NumCopiesOfFat      equ    10h    ; DB   # of fats
RootDirEnteries     equ    11h    ; DW   # of root dir entries
SectorsTo32         equ    13h    ; DW   # sectors total in image
MediaDescriptor     equ    15h    ; DB   media descrip: fd=2side9sec, etc...
SectorsPerFat       equ    16h    ; DW   # sectors in a fat
SectorsPerTrack     equ    18h    ; DW   # sectors/track
NumberOfHeads       equ    1Ah    ; DW   # heads
HiddenSectors       equ    1Ch    ; DW   # hidden sectors
SectorsAbove32      equ    20h    ; DD   # sectors if > 65536
BootDriveNumber     equ    24h    ; DB   drive number (A:=0, B:=1,C:=2)
HeadNumber          equ    25h    ; DB   also listed as reserved byte
ExtBootSig          equ    26h    ; DB   extended boot signature
;----------------------------------------------------------------------------
;
; These 15 byte can be altered to identify each intervidual disk.
;
; reserved for      equ    27h    ; DD   ID number for each intervidual disk
; reserved for      equ    2Bh    ; 11 bytes for end user, use as disk ID
;----------------------------------------------------------------------------
;
; Disks FAT ID (either fat12, fat16 or fat32) entered during formating.
;
;                   equ    36h    ; 8 bytes identifies the disks format
;----------------------------------------------------------------------------

; These last 16 bytes are used by freedos's  boot sector as buffer and
; storage area and are not part of the standard dos data.

tempbuf             equ    3Eh    ; DW   
RootDirSecs         equ    40h    ; DW   
fat_start           equ    42h    ; DD   
root_dir_start      equ    46h    ; DD
data_start          equ    4Ah    ; DD
;-----------------------------------------------------------------------

       org     0                  ; this become 7C00h on startup
Entry:
       jmp     short real_start
       db      90h
       db      'F-BOOT.0'

%define LOADSEG         0x0060
%define FATBUF          0x2000    ; temporary buffer for FAT chain

		times   0x3E-$+$$ db 0

%define tempbuf         0x3E
		dw      LOADSEG

%define RootDirSecs     0x40      ; # of sectors root dir uses
		dw      FATBUF
%define fat_start       0x42      ; first FAT sector
		dd      0
%define root_dir_start  0x46      ; first root directory sector
		dd      0
%define data_start      0x4A      ; first data sector
		dd      0
;-----------------------------------------------------------------------

real_start:
       cli
       xor     ax,ax
       mov     ss,ax
       mov     ds,ax
       mov     bp,7C00h
       mov     sp,7BE0h
       sti
       int     12h
       mov     ax,1FE0h
       mov     es,ax
       mov     si,bp
       mov     di,bp
       mov     cx,100h
       rep     movsw
       push    es
       mov     bx,cont + 7C00h
       push    bx
       retf
cont:
       mov     ds,ax
       mov     ss,ax
       call    print              ; old freedos method of printing message
       db      0Dh,0Ah
       db      'Loading FreeDOS...', 0Dh,0Ah,00h
       mov     ax,word [bp+root_dir_start]
       mov     dx,word [bp+root_dir_start+2]
       div     word [bp+SectorsPerTrack]
       mov     cx,dx
       xor     dx,dx
       div     word [bp+NumberOfHeads]
       ror     ah,1
       ror     ah,1
       mov     dh,dl
       inc     cx
       and     ah,0Ch
       mov     dl,byte [bp+BootDriveNumber]
       xchg    al,ah
       or      cx,ax
       mov     ax,201h
       int     13h
       jc      error_exit 
;---------------------------------------------
       mov     bx,60h
       mov     es,bx
       xor     bx,bx
       mov     di,[bp+SectorsPerFat]
       mov     ax,word [bp+fat_start]
       mov     dx,word [bp+fat_start+2]
;---------------------------------------------
       call    readDisk
       push    ds
       pop     es
       mov     dx,60h
       xor     di,di
next_file:
       mov     cx,0Bh
       mov     si,next + 7C00h
       push    di
       repe    cmpsb
       pop     di
       jz      kernel_file_found
       add     di,20h
       cmp     byte [es:di],0
       jne     next_file
error_exit:
       jmp     boot_error
kernel_file_found:
       mov     ax,[es:di+1Ah] 
       mov     di,2000h
       mov     ds,dx 
       add     dh,10h
next_clust:
       stosw
       mov     si,ax
       add     si,si
       jnc     fat_16
       mov     ds,dx
fat_16:
       lodsw
       cmp     ax,0FFF8h
       jb      next_clust
       mov     si,2000h
       xor     ax,ax
       stosw
       mov     bx,60h
       push    cs
       pop     ds
       mov     es,bx
       xor     bx,bx
readDisk3:
       lodsw
       or      ax,ax
       jz      boot_success
       dec     ax
       mov     di,[bp+SectorsPerCluster]
       and     di,0FFh
       dec     ax
       mul     di
       add     ax,word [bp+data_start]
       adc     dx,word [bp+data_start+2]
       call    readDisk
       jmp     short readDisk3
boot_success:
       mov     bl,byte [bp+BootDriveNumber]
       db      0EAh                   ; jmp far 0060:0000h
       dw      0, 60h
;-----------------------------------------------------------
readDisk:
       push    dx
       push    ax
       div     word [bp+SectorsPerTrack]
       mov     cx,dx
       xor     dx,dx
       div     word [bp+NumberOfHeads]
       ror     ah,1
       ror     ah,1
       mov     dh,dl
       inc     cx
       and     ah,0Ch
       mov     dl,byte [bp+BootDriveNumber]
       xchg    al,ah
       or      cx,ax
       mov     ax,201h
       int     13h
       jc      error
       add     bx,word [bp+BytesPerSector]
       jnc     readDisk_1
       mov     ax,es
       add     ah,10h
       mov     es,ax
readDisk_1:
       pop     ax
       pop     dx
       add     ax,1
       adc     dx,0
       dec     di
       jnz     readDisk
       ret
error:
       pop     word [si]
;----------------------------------------------------
boot_error:
       call    print
       db      0Dh, 0Ah
       db      'Remove Non-Boot Disk in disk drive'
       db      0Dh, 0Ah
       db      'and then press any key to continue',0
       db      0Dh, 0Ah, 0
       mov     ah,0
       int     16h
       int     19h
       pop     si
       pop     ds
       pop     word [si]
       jmp     short boot_error
;-----------------------------------------------------------
print:
       pop     si
       xor     bx,bx
       mov     ah,0Eh
print_1:
       cmp     al,0
       je      print_2
       lodsb
       int     10h
       jmp     print_1
print_2:
       push    si
       ret
;-----------------------------------------------------
next   db      'KERNEL  SYS'
       times   0x01fe-$+$$ db 0
       db      55h,0AAh
       end 
